// Generated by CoffeeScript 1.10.0
(function() {
  var KicadGenerator, QedaElement, QedaLibrary, fs, log, mkdirp, path, request, yaml;

  fs = require('fs');

  mkdirp = require('mkdirp');

  path = require('path');

  request = require('sync-request');

  yaml = require('js-yaml');

  KicadGenerator = require('./kicad-generator');

  QedaElement = require('./qeda-element');

  log = require('./qeda-log');

  QedaLibrary = (function() {
    function QedaLibrary(config) {
      if (config == null) {
        config = {};
      }
      this.output = 'kicad';
      this.connection = {
        timeout: 5000
      };
      this.symbol = {
        style: 'default',
        gridSize: 2.5,
        fontSize: {
          "default": 2.5,
          refDes: 2.5,
          name: 2.5,
          pin: 2.5
        },
        lineWidth: {
          "default": 0,
          thick: 0.6,
          thin: 0.2
        },
        space: {
          "default": 2,
          pin: 2,
          attribute: 1.5
        }
      };
      this.pattern = {
        style: 'default',
        densityLevel: 'N',
        decimals: 3,
        polarityMark: 'dot',
        preferManufacturer: true,
        tolerance: {
          "default": 0.1,
          fabrication: 0.1,
          placement: 0.1
        },
        clearance: {
          padToSilk: 0.2,
          padToPad: 0.2,
          padToMask: 0.05,
          holeOverLead: 0.2
        },
        ratio: {
          padToHole: 1.5
        },
        minimum: {
          ringWidth: 0.2,
          drillDiameter: 0.2
        },
        lineWidth: {
          "default": 0.2,
          silkscreen: 0.12,
          assembly: 0.1,
          courtyard: 0.05
        },
        fontSize: {
          "default": 1,
          refDes: 1.2,
          value: 1
        },
        ball: {
          collapsible: true
        }
      };
      this.mergeObjects(this, config);
      this.elements = [];
    }

    QedaLibrary.prototype.add = function(element) {
      var alias, aliases, base1, def, housing, housings, j, key, keys, l, len, len1, len2, m, name, newElement, ref, res, suffixes;
      def = this.load(element);
      if (def.abstract) {
        console.error("'" + element + "': Cannot add abstract component, use it only as base for others");
        process.exit(1);
      }
      res = [];
      housings = [];
      if (typeof def.housing === 'string') {
        keys = def.housing.replace(/\s+/g, '').split(',');
        for (j = 0, len = keys.length; j < len; j++) {
          key = keys[j];
          housing = def[key];
          if (housing.suffix == null) {
            housing.suffix = key;
          }
          housings.push(housing);
        }
      } else {
        if ((base1 = def.housing).suffix == null) {
          base1.suffix = '';
        }
        housings.push(def.housing);
      }
      name = def.name;
      for (l = 0, len1 = housings.length; l < len1; l++) {
        housing = housings[l];
        suffixes = housing.suffix.replace(/\s+/g, '').split(',');
        def.name = name + suffixes[0];
        def.housing = housing;
        aliases = [];
        if (def.alias != null) {
          ref = def.alias.replace(/\s+/g, '').split(',');
          for (m = 0, len2 = ref.length; m < len2; m++) {
            alias = ref[m];
            aliases = aliases.concat(suffixes.map((function(_this) {
              return function(v) {
                return alias + v;
              };
            })(this)));
          }
        }
        if (suffixes.length > 1) {
          aliases = aliases.concat(suffixes.slice(1).map((function(_this) {
            return function(v) {
              return name + v;
            };
          })(this)));
        }
        def.aliases = aliases;
        newElement = new QedaElement(this, def);
        this.elements.push(newElement);
        res.push(newElement);
      }
      return res;
    };

    QedaLibrary.prototype.addSymbol = function(symbol, name) {
      var def, newElement;
      def = {
        name: name,
        schematic: {
          symbol: symbol
        }
      };
      newElement = new QedaElement(this, def);
      this.elements.push(newElement);
      return newElement;
    };

    QedaLibrary.prototype.generate = function(name) {
      var generator;
      log.start("Render library '" + name + "'");
      this.render();
      log.ok();
      generator = null;
      switch (this.output) {
        case 'kicad':
          generator = new KicadGenerator(this);
      }
      if (generator) {
        log.start("Generate output for '" + name + "'");
        generator.generate(name);
        return log.ok();
      }
    };

    QedaLibrary.prototype.ground = function(name) {
      return this.addSymbol('ground', name);
    };

    QedaLibrary.prototype.load = function(element, force) {
      var base, baseDef, baseElement, bases, def, e, elementYaml, error, error1, error2, exclude, j, l, len, len1, localFile, res;
      if (force == null) {
        force = false;
      }
      elementYaml = element.toLowerCase() + '.yaml';
      localFile = './library/' + elementYaml;
      if ((!fs.existsSync(localFile)) || force) {
        log.start("Load '" + element + "'");
        elementYaml = elementYaml.split('/').map(function(a) {
          return encodeURIComponent(a);
        }).join('/');
        try {
          res = request('GET', 'https://raw.githubusercontent.com/qeda/library/master/' + elementYaml, {
            timeout: this.connection.timeout
          });
        } catch (error1) {
          error = error1;
          log.error(error.message);
        }
        if (res.statusCode === 200) {
          mkdirp.sync(path.dirname(localFile));
          fs.writeFileSync(localFile, res.body);
          log.ok();
        } else {
          log.warning(res.statusCode);
        }
      }
      log.start("Read '" + element + "'");
      try {
        def = yaml.safeLoad(fs.readFileSync(localFile));
      } catch (error2) {
        error = error2;
        log.error("" + error.message);
      }
      if (def.base != null) {
        if (typeof def.base !== 'string') {
          def.base = def.base.toString();
        }
        bases = def.base.replace(/\s+/g, '').split(',');
        delete def.base;
        exclude = ['abstract', 'alias'];
        for (j = 0, len = bases.length; j < len; j++) {
          base = bases[j];
          baseElement = base;
          if (path.dirname(baseElement) === '.') {
            baseElement = path.dirname(element) + '/' + baseElement;
          }
          baseDef = this.load(baseElement);
          for (l = 0, len1 = exclude.length; l < len1; l++) {
            e = exclude[l];
            if (baseDef[e] != null) {
              delete baseDef[e];
            }
          }
          this.mergeObjects(baseDef, def);
          def = baseDef;
        }
      }
      log.ok();
      return def;
    };

    QedaLibrary.prototype.mergeObjects = function(dest, src) {
      var k, results, v;
      results = [];
      for (k in src) {
        v = src[k];
        if (typeof v === 'object' && (!Array.isArray(v)) && dest.hasOwnProperty(k)) {
          results.push(this.mergeObjects(dest[k], v));
        } else {
          results.push(dest[k] = v);
        }
      }
      return results;
    };

    QedaLibrary.prototype.power = function(name) {
      return this.addSymbol('power', name);
    };

    QedaLibrary.prototype.render = function() {
      var baseName, element, i, j, keys, len, name, patterns, ref;
      if (this._rendered == null) {
        this._rendered = false;
      }
      if (this._rendered) {
        return;
      }
      patterns = {};
      ref = this.elements;
      for (j = 0, len = ref.length; j < len; j++) {
        element = ref[j];
        element.render();
        if (element.pattern != null) {
          baseName = element.pattern.name;
          name = baseName;
          keys = Object.keys(patterns);
          i = 1;
          while (keys.indexOf(name) !== -1) {
            if (element.pattern.isEqualTo(patterns[name])) {
              break;
            } else {
              name = baseName + "-" + (i++);
              element.pattern.name = name;
            }
          }
          patterns[name] = element.pattern;
        }
      }
      return this._rendered = true;
    };

    return QedaLibrary;

  })();

  module.exports = QedaLibrary;

}).call(this);
